      SUBROUTINE HDIAG(H,N,IEGEN,U,NR,X,IQ)
C**********************************************************************
C*    HDIAG - DIAGONALIZATION OF A REAL SYMMETRIC MATRIX.
C*          USES THE JACOBI METHOD
C*    CALLING SEQUENCE FOR DIAGONALIZATION
C*          CALL  HDIAG( H, N, IEGEN, U, NR,NDIM)
C*           WHERE H IS THE ARRAY TO BE DIAGONALIZED.
C*    N IS THE ORDER OF THE MATRIX, H.
C*
C*    IEGEN MUST BE SET UNEQUAL TO ZERO IF ONLY EIGENVALUES ARE
C*           TO BE COMPUTED.
C*    IEGEN MUST BE SET EQUAL TO ZERO IF EIGENVALUES AND EIGENVECTORS
C*           ARE TO BE COMPUTED.
C*
C*    U IS THE UNITARY MATRIX USED FOR FORMATION OF THE EIGENVECTORS.
C*
C*    NR IS THE NUMBER OF ROTATIONS.
C*    X  = WORKING STORAGE
C*    IQ = WORKING STORAGE
C*    NDIM = SIZE OF ARRAYS IN MAIN PROGRAM DIMENSION STATEMENT
C*
C*    THE SUBROUTINE OPERATES ONLY ON THE ELEMENTS OF H THAT ARE TO THE
C*           RIGHT OF THE MAIN DIAGONAL.  THUS, ONLY A TRIANGULAR
C*           SECTION NEED BE STORED IN THE ARRAY H.
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION H(3,3),U(3,3),X(3),IQ(3)
C**********************************************************************
C*    SET INDICATOR FOR SHUT-OFF.
C**********************************************************************
C      write(*,*) 'in hdiag'
      HDTEST=1.0D30
      RAP=2.0**(-48)
      IF(IEGEN.NE.0) GOTO15
 10   DO 14 I=1,N
        DO 14 J=1,N
          IF(I.EQ.J) GOTO 11
          GOTO 12
 11       U(I,J)=1.0
          GOTO 14
 12       U(I,J)=0.
 14       CONTINUE
 15   NR=0
      IF(N.LE.1) GOTO 1000
C********************************************************************
C*    SCAN FOR LARGEST OFF DIAGONAL ELEMENT IN EACH ROW
C*    X(I) CONTAINS LARGEST ELEMENT IN ITH ROW
C*    IQ(I) HOLDS SECOND SUBSCRIPT DEFINING POSITION OF ELEMENT
C********************************************************************
 17   NMI1=N-1
      DO 30 I=1,NMI1
        X(I)=0.
        IPL1=I+1
        DO 30 J=IPL1,N
          IF (X(I)-DABS(H(I,J))) 20,20,30
 20       X(I)=DABS(H(I,J))
          IQ(I)=J
 30       CONTINUE
C********************************************************************
C*    FIND MAXIMUM OF X(I) S FOR PIVOT ELEMENT AND
C*    TEST FOR END OF PROBLEM
C********************************************************************
 40   DO 70 I=1,NMI1
        IF (I.LE.1) GOTO 60
   45   IF (XMAX.GE.X(I)) GOTO 70
 60     XMAX=X(I)
        IPIV=I
        JPIV=IQ(I)
 70     CONTINUE
C********************************************************************
C*    IS MAX. X(I) EQUAL TO ZERO, IF LESS THAN HDTEST, REVISE HDTEST
C********************************************************************
      IF (XMAX) 1000,1000,80
 80   IF (HDTEST) 90,90,85
 85   IF (XMAX-HDTEST) 90,90,148
 90   HDIMIN=DABS(H(1,1))
      DO 110 I=2,N
        IF (HDIMIN-DABS(H(I,I))) 110,110,100
 100    HDIMIN=DABS(H(I,I))
 110    CONTINUE
      HDTEST=HDIMIN*RAP
C********************************************************************
C*    RETURN IF MAX.H(I,J)LESS THAN(2**-48)ABS(H(K,K)-MIN)
C********************************************************************
      IF (HDTEST-XMAX) 148,1000,1000
 148  NR=NR+1
C********************************************************************
C*    COMPUTE TANGENT, SINE AND COSINE,H(I,I),H(J,J)
C********************************************************************
 150  TANG=DSIGN(2.0D0,(H(IPIV,IPIV)-H(JPIV,JPIV)))*
     +H(IPIV,JPIV)/(ABS(H(IPIV,IPIV)-H(JPIV,JPIV))+
     +SQRT((H(IPIV,IPIV)-H(JPIV,JPIV))**2+4.0*H(IPIV,JPIV)**2))
      COSINE=1.0/DSQRT(1.0+TANG**2)
      SINE=TANG*COSINE
      HII=H(IPIV,IPIV)
      H(IPIV,IPIV)=COSINE**2*(HII+TANG*(2.*H(IPIV,JPIV)+
     +TANG*H(JPIV,JPIV)))
      H(JPIV,JPIV)=COSINE**2*(H(JPIV,JPIV)-
     +TANG*(2.*H(IPIV,JPIV)-TANG*HII))
      H(IPIV,JPIV)=0.
C********************************************************************
C*     PSEUDO RANK THE EIGENVALUES
C*     ADJUST SINE AND COS FOR COMPUTATION OF H(IK) AND U(IK)
C********************************************************************
      IF (H(IPIV,IPIV)-H(JPIV,JPIV)) 152,153,153
 152  HTEMP=H(IPIV,IPIV)
      H(IPIV,IPIV)=H(JPIV,JPIV)
      H(JPIV,JPIV)=HTEMP
C*****RECOMPUTE SINE AND COS
      HTEMP=DSIGN(1.0D0,-SINE)*COSINE
      COSINE=DABS(SINE)
      SINE=HTEMP
 153  CONTINUE
C********************************************************************
C*    INSPECT THE IQS BETWEEN I+1 AND N-1 TO DETERMINE
C*    WHETHER A NEW MAXIMUM VALUE SHOULD BE COMPUTED SINCE
C*    THE PRESENT MAXIMUM IS IN THE I OR J ROW.
C********************************************************************
      DO 350 I=1,NMI1
        IF (I-IPIV) 210,350,200
 200    IF (I-JPIV) 210,350,210
 210    IF (IQ(I)-IPIV) 230,240,230
 230    IF (IQ(I)-JPIV) 350,240,350
 240    K=IQ(I)
 250    HTEMP=H(I,K)
        H(I,K)=0.
        IPL1=I+1
        X(I)=0.
C********************************************************************
C*      SEARCH IN DEPLETED ROW FOR NEW MAXIMUM
C********************************************************************
        DO 320 J=IPL1,N
          IF (X(I)-DABS(H(I,J))) 300,300,320
 300      X(I)=DABS(H(I,J))
          IQ(I)=J
 320      CONTINUE
        H(I,K)=HTEMP
 350    CONTINUE
      X(IPIV) =0.
      X(JPIV) =0.
C********************************************************************
C*    CHANGE THE OTHER ELEMENTS OF H
C********************************************************************
      DO 530 I=1,N
        IF (I-IPIV) 370,530,420
 370    HTEMP=H(I,IPIV)
        H(I,IPIV)=COSINE*HTEMP+SINE*H(I,JPIV)
        IF (X(I)-DABS(H(I,IPIV))) 380,390,390
 380    X(I)=DABS(H(I,IPIV))
        IQ(I)=IPIV
 390    H(I,JPIV)=-SINE*HTEMP+COSINE*H(I,JPIV)
        IF (X(I)-DABS(H(I,JPIV))) 400,530,530
 400    X(I)=DABS(H(I,JPIV))
        IQ(I)=JPIV
        GO TO 530
 420    IF (I-JPIV) 430,530,480
 430    HTEMP=H(IPIV,I)
        H(IPIV,I)=COSINE*HTEMP+SINE*H(I,JPIV)
        IF (X(IPIV)-DABS(H(IPIV,I))) 440,450,450
 440    X(IPIV)=DABS(H(IPIV,I))
        IQ(IPIV)=I
 450    H(I,JPIV)=-SINE*HTEMP+COSINE*H(I,JPIV)
        IF (X(I)-DABS(H(I,JPIV))) 400,530,530
 480    HTEMP=H(IPIV,I)
        H(IPIV,I)=COSINE*HTEMP+SINE*H(JPIV,I)
        IF (X(IPIV)-DABS(H(IPIV,I))) 490,500,500
 490    X(IPIV)=DABS(H(IPIV,I))
        IQ(IPIV)=I
 500    H(JPIV,I)=-SINE*HTEMP+COSINE*H(JPIV,I)
        IF (X(JPIV)-DABS(H(JPIV,I))) 510,530,530
 510    X(JPIV)=DABS(H(JPIV,I))
        IQ(JPIV)=I
 530    CONTINUE
C********************************************************************
C*    TEST FOR COMPUTATION OF EIGENVECTORS
C********************************************************************
      IF (IEGEN.EQ.0) GOTO 540
      GOTO 40
 540  DO 550 I=1,N
        HTEMP=U(I,IPIV)
        U(I,IPIV)=COSINE*HTEMP+SINE*U(I,JPIV)
 550    U(I,JPIV)=-SINE*HTEMP+COSINE*U(I,JPIV)
      GOTO 40
 1000 RETURN
      END
